@@ -1,3 +1,7 @@
+/* Debug flags */
+#undef _DEBUG
+#undef CHECK_RANGE
+
 /* Define if the C++ compiler supports BOOL */
 #undef HAVE_BOOL
 
@@ -13,3 +17,10 @@
 
 #undef HAVE_TERMIOS_FUNCS
 
+#undef VA_COPY
+
+#undef INSTALL_PREFIX
+
+#undef INSTALL_DATA_DIR
+
+#undef HAVE_READLINE_42
