@@ -1,10 +1,24 @@
 /***************************************************************************
-                          byteorder.cpp  -  description
+ Mutella - A commandline/HTTP client for the Gnutella filesharing network.
+
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License
+ as published by the Free Software Foundation; either version 2
+ of the License, or (at your option) any later version.
+
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+
+ byteorder.cpp  -  Bit/Byte order management class definition
                              -------------------
     begin                : Thu Jun 8 2000
     copyright            : (C) 2000 by Max Zaitsev
     email                : maksik@gmx.co.uk
  ***************************************************************************/
+ 
+ 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -12,8 +26,6 @@
 #include <stdlib.h>
 #include "byteorder.h"
 
-byteOrderTest MByteOrderTest::ms_ByteOrderTest = {1};
-
 void BitReorder(unsigned char* pBuffer, int nSize, int nElementSize, bool bBits, bool bBytes)
 {
 	if (bBits|bBytes)
@@ -51,3 +63,4 @@ void BitReorder(unsigned char* pBuffer, int nSize, int nElementSize, bool bBits,
 		}
 	}		
 }
+
