@@ -1,26 +1,46 @@
 /***************************************************************************
-                          gnuupload.cpp  -  description
-                             -------------------
+ Mutella - A commandline/HTTP client for the Gnutella filesharing network.
+
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License
+ as published by the Free Software Foundation; either version 2
+ of the License, or (at your option) any later version.
+
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+
+ gnuupload.cpp  -  File upload controller class
+ 
+ the original version of this file was taken from Gnucleus (http://gnucleus.sourceforge.net)
+
     begin                : Tue May 29 2001
     copyright            : (C) 2001 by
     email                : maksik@gmx.co.uk
  ***************************************************************************/
 
-// the original version of this file was taken from Gnucleus (http://gnucleus.sourceforge.net)
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include "mutella.h"
-#include "controller.h"
+#include "structures.h"
 
+#include "asyncsocket.h"
+#include "packet.h"
+
+#include "event.h"
+#include "messages.h"
 #include "gnushare.h"
-#include "gnuhash.h"
 #include "gnudirector.h"
-#include "preferences.h"
 
 #include "gnuupload.h"
+#include "conversions.h"
+#include "common.h"
+#include "property.h"
+#include "preferences.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -30,51 +50,47 @@
 
 #define DEF_UPLOAD_BUFFER 0x00004000
 
-/////////////////////////////////////////////////////////////////////////////
-// service: we need this set for thread-safe callbacks
-
-static MMutex s_mutexUploadSet(true); // recursive to enable 'delete this' from callbacks
-static set<MGnuUpload*> s_setUploads;
-//
-static inline void RegisterUpload(MGnuUpload* p)
-{
-	s_mutexUploadSet.lock();
-	ASSERT(s_setUploads.find(p)==s_setUploads.end());
-	s_setUploads.insert(p);
-	s_mutexUploadSet.unlock();
-}
-static inline void UnregisterUpload(MGnuUpload* p)
-{
-	s_mutexUploadSet.lock();
-	ASSERT(s_setUploads.find(p)!=s_setUploads.end());
-	s_setUploads.erase(p);
-	s_mutexUploadSet.unlock();
-}
-static inline bool IsRegistred(MGnuUpload* p)
-{
-	//MLock lock(s_mutexUploadSet); // it is always called with mutex locked
-	return s_setUploads.find(p)!=s_setUploads.end();
-}
-
 //////////////////////////////////////////////////////////////////////////////
 class MUploadFile : public MAsyncFile
 {
 public:
 	MUploadFile(MGnuUpload* pUpload);
-	virtual void OnSuccess();
-	virtual void OnError();
-	void GiveAway(){
-		s_mutexUploadSet.lock();
-		m_pBuddy = NULL;
-		s_mutexUploadSet.unlock();
-	}
+	virtual void OnSuccessDelayed(DWORD dwSeqReqID, int nReqType, const SAFileState& FileState);
+	virtual void OnErrorDelayed(DWORD dwSeqReqID, int nReqType, const SAFileState& FileState);
 protected:
 	MGnuUpload* m_pBuddy;
 private:
 	virtual ~MUploadFile();
 };
 
-MUploadFile::MUploadFile(MGnuUpload* pBuddy) : MAsyncFile(AFM_READ, DEF_UPLOAD_BUFFER), m_pBuddy(pBuddy)
+class MUploadBuffer : public MAFBufferTrivial
+{
+public:
+	MUploadBuffer(int nBufferSize = 16384) : MAFBufferTrivial(nBufferSize) {
+		m_nContains = 0;
+		m_nOffset = 0;
+	}
+	// TODO: mutex-protect it all
+	inline char* data(){return m_pBuffer + m_nOffset;}
+	inline char* space(){ if (remains()>0) return m_pBuffer + m_nContains; return NULL; }
+	inline int   contains(){return m_nContains - m_nOffset;}
+	inline int   size(){return m_nSize - m_nOffset;}
+	inline int   remains(){return m_nSize - m_nContains;}
+	inline bool  increment(int nBy){ASSERT(nBy>=0); ASSERT(m_nContains>=0); m_nContains += nBy; ASSERT(m_nContains <= m_nSize); return true;}
+	inline bool  isFull(){return m_nContains == m_nSize;}
+	inline bool  isEmpty(){return m_nContains==0;}
+	inline void  clear(){m_nContains = 0; m_nOffset = 0;}
+	inline bool  offset(int nBy) {ASSERT(nBy>=0); ASSERT(m_nOffset>=0); m_nOffset += nBy; ASSERT(m_nOffset <= m_nSize); return true;}
+protected:
+	int    m_nContains;
+	int    m_nOffset;
+	//
+	~MUploadBuffer(){}
+};
+
+////////////////////////////////////////////////////////////////////////////////
+
+MUploadFile::MUploadFile(MGnuUpload* pBuddy) : MAsyncFile(AFM_READ), m_pBuddy(pBuddy)
 {
 }
 
@@ -82,50 +98,35 @@ MUploadFile::~MUploadFile()
 {
 }
 
-void MUploadFile::OnSuccess()
+void MUploadFile::OnSuccessDelayed(DWORD dwSeqReqID, int nReqType, const SAFileState& FileState)
 {
-	//TRACE2("MUploadFile::OnSuccess; request=", m_pRequest->type);
-	s_mutexUploadSet.lock();
-	//ASSERT(m_pThread==s_context.pAFThreadRead || m_pThread==s_context.pAFThreadWrite);
-	//
-	switch (m_pRequest->type_compl)
+	switch (nReqType)
 	{
 		case AFR_OPEN:
-			if (m_pBuddy && IsRegistred(m_pBuddy))
+			if (m_pBuddy)
 				m_pBuddy->OnFileOpen();
 		break;
 		case AFR_READ:
-		case AFR_PREREAD:
-			if (m_pBuddy && IsRegistred(m_pBuddy))
-				m_pBuddy->OnFileRead(m_pRequest->size, m_pRequest->nReturn, m_pRequest->nErrNo);
+		case AFR_READ|AFR_SEEK:
+			if (m_pBuddy)
+				m_pBuddy->OnFileRead(FileState.nSize, FileState.nReturn, FileState.nErrNo);
 		break;
 		case AFR_NONE:
 			TRACE("MUploadFile::OnSuccess: AFR_NONE - should not happen");
 		break;
 	}
-	//
-	s_mutexUploadSet.unlock();
 }
 
-void MUploadFile::OnError()
+void MUploadFile::OnErrorDelayed(DWORD dwSeqReqID, int nReqType, const SAFileState& FileState)
 {
-	//TRACE2("MUploadFile: on error; request=",m_pRequest->type_compl);
-	s_mutexUploadSet.lock();
-	//ASSERT(m_pThread==s_context.pAFThreadRead || m_pThread==s_context.pAFThreadWrite);
-	//
-	if (m_pBuddy && IsRegistred(m_pBuddy))
-		m_pBuddy->OnFileError(m_pRequest->nErrNo);
-	if (m_pRequest->type_compl == AFR_CLOSE)
-		delete this;
-	s_mutexUploadSet.unlock();
+	if (m_pBuddy)
+		m_pBuddy->OnFileError(FileState.nErrNo);
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 MGnuUpload::MGnuUpload(MGnuDirector* pComm)
 {
-	RegisterUpload(this);
-	//
 	MLock lock(m_mutex);
 	//
 	m_pDirector  = pComm;
@@ -133,13 +134,24 @@ MGnuUpload::MGnuUpload(MGnuDirector* pComm)
 	m_pPrefs = m_pDirector->GetPrefs();
 
 	m_sFileName = "Unknown";
+	m_sUserAgent = "Unknown";
 	m_nFileLength = 0;
+	m_nRequestedRange = -1;
 	m_pAFile = new MUploadFile(this);
+	ASSERT(m_pAFile);
+	m_pBuffer = new MUploadBuffer(4096); // 4k
+	m_pBuffer->Release(); // SmartPtr takes responsibility
 
 	m_nBytesCompleted = 0;
 	m_nFileIndex = -1;
 	m_nFileIndexPush = -1;
 
+	m_bUriRequest = false;
+
+	m_nStartOffset = 0;
+	m_nStartTime = xtime();
+	m_bHttpOK = false; // we have not yet accepted the transfer
+
 	// Bandwidth
 	for(int i = 0; i < 60; i++)
 		m_dwAvgBytes[i] = 0;
@@ -170,8 +182,6 @@ MGnuUpload::MGnuUpload(MGnuDirector* pComm)
 
 MGnuUpload::~MGnuUpload()
 {
-	//TRACE("~MGnuUpload");
-	UnregisterUpload(this);
 	//
 	m_mutex.lock();
 	
@@ -179,12 +189,35 @@ MGnuUpload::~MGnuUpload()
 	
 	if( m_pAFile )
 	{
-		((MUploadFile*)m_pAFile)->GiveAway();
 		m_pAFile->Close();
-		m_pAFile->Destroy(); // will call 'delete this'
+		delete m_pAFile;
 		m_pAFile = NULL;
 	}
 	m_mutex.unlock();
+
+	if (m_bHttpOK)
+	{
+		// this message is only needed for logging
+		// if we intend to use it for anything else we'll have to
+		// define a custom event type
+		CString s;
+		int nRelTime = xtime() - m_nStartTime;
+		s.format("start:%s\t pos:%d\t sent:%d\t speed:%sb/s\t host:%s\t client:%s\t file:%s",
+				FormatAbsTimeFull(m_nStartTime).c_str(),
+				m_nStartOffset,
+				m_nBytesCompleted-m_nStartOffset,
+				nRelTime>0 ? FormatSize((m_nBytesCompleted-m_nStartOffset)/nRelTime).c_str() : "--.-",
+				Ip2Str(m_ipHost).c_str(),
+				m_sUserAgent.c_str(),
+				m_sFileName.c_str());
+		POST_EVENT( MStringEvent(
+			ET_MESSAGE,
+			ES_MINIMAL,
+			s,
+			MSG_UPLOAD_FINISHED,
+			MSGSRC_UPLOAD
+		));
+	}
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -255,6 +288,7 @@ void MGnuUpload::OnReceive(int nErrorCode)
 	MLock lock(m_mutex);
 	
 	BYTE* pBuff = new BYTE[6000];
+	ASSERT(pBuff);
 	DWORD dwBuffLength = Receive(pBuff, 4096);
 	
 	switch (dwBuffLength)
@@ -287,7 +321,8 @@ void MGnuUpload::OnReceive(int nErrorCode)
 	{
 		CString Handshake = Header;
 
-		if(Handshake.find("GET /get/") == 0)
+		if( Handshake.find("GET /get/") == 0                  ||
+			Handshake.find("GET /uri-res/N2R?urn:sha1:") == 0 )
 		{
 			// Get Node info
 			CString Host;
@@ -295,9 +330,9 @@ void MGnuUpload::OnReceive(int nErrorCode)
 			GetPeerName(Host, nPort);
 			
 			// Set Variables
-			m_ipHost       = StrtoIP(Host.c_str());
-			m_nPort        = 0;
-			// unlock/lock here doesnt look elegant, but we heed it
+			Str2Ip(Host,m_ipHost);
+			m_nPort = nPort;
+			// "false" means that the mutex is locked
 			UploadFile(Handshake,false);
 		}
 		else
@@ -313,20 +348,32 @@ void MGnuUpload::OnReceive(int nErrorCode)
 }
 
 void MGnuUpload::Send_HttpOK()
-{	
-	CString sLength = DWrdtoStr(m_nFileLength);
+{
+	CString HttpOK;
+	if(m_nBytesCompleted || m_nRequestedRange != m_nFileLength)
+		HttpOK =  "HTTP/1.1 206 Partial Content\r\n";
+	else
+		HttpOK =  "HTTP/1.0 200 OK\r\n";
+	HttpOK += "Server: Mutella-";
+	HttpOK += VERSION;
+	HttpOK += "\r\n";
+	HttpOK += "Content-Type: application/binary\r\n";
+	if(m_nBytesCompleted || m_nRequestedRange != m_nFileLength)
+	{
+		HttpOK += "Accept-Ranges: bytes\r\n";
+		HttpOK += "Content-range: bytes ";
+		HttpOK +=  DWrdtoStr(m_nBytesCompleted) + "-" + DWrdtoStr(m_nRequestedRange - 1) + "/" + DWrdtoStr(m_nFileLength) + "\r\n";
+	}
 
-	if(m_nBytesCompleted)
-		sLength = "BYTEs=" + DWrdtoStr(m_nBytesCompleted) + "-" + DWrdtoStr(m_nFileLength - 1) + "/" + DWrdtoStr(m_nFileLength);
+	HttpOK += "Content-Length: " + DWrdtoStr(m_nRequestedRange - m_nBytesCompleted) + "\r\n";
 
-	CString HttpOK =  "HTTP 200 OK\r\n";
-			HttpOK += "Server: Mutella\r\n";
-			HttpOK += "Content-type:application/binary\r\n";
-			if(m_nBytesCompleted)
-				HttpOK += "Accept-Ranges: BYTEs\r\nContent-range: " + sLength + "\r\n";
-			else
-				HttpOK += "Content-length: " + sLength + "\r\n";
-			HttpOK += "\r\n";
+	// X-Filename or X-Gnutella-Content-URN
+	if(m_bUriRequest)
+		HttpOK += "X-Filename: " + m_sFileName + "\r\n";
+	else if (!m_sSha1.empty())
+		HttpOK += "X-Gnutella-Content-URN: urn:sha1:" + m_sSha1 + "\r\n";
+	
+	HttpOK += "\r\n";
 	Send(HttpOK.c_str(), HttpOK.length());
 
 	StatusUpdate(TRANSFER_SENDING);
@@ -335,7 +382,7 @@ void MGnuUpload::Send_HttpOK()
 
 void MGnuUpload::Send_HttpBusy()
 {
-	CString Http503 =  "HTTP 503\r\n";
+	CString Http503 =  "HTTP/1.0 503\r\n";
 			Http503 += "Server: Mutella\r\n";
 			Http503 += "Content-type:text/html\r\n\r\n";
 			Http503 += "<HTML>\r\n";
@@ -357,14 +404,14 @@ void MGnuUpload::Send_HttpBusy()
 
 void MGnuUpload::Send_HttpNotFound()
 {
-	CString Http404 =  "HTTP 404\r\n";
+	CString Http404 =  "HTTP/1.0 404\r\n";
 			Http404 += "Server: Mutella\r\n";
 			Http404 += "Content-type:text/html\r\n\r\n";
 			Http404 += "<HTML>\r\n";
 			Http404 += "<HEAD><TITLE>404 Not Found</TITLE></HEAD>\r\n";
 			Http404 += "<BODY>\r\n";
 			Http404 += "<H1>Not Found</H1>\r\n";
-			Http404 += "The requested file \"<I>" + m_sFileName + "\"</I> was not found on this server.\r\n";
+			Http404 += "The requested file \"<I>" + m_sFileName + "</I>\" was not found on this server.\r\n";
 			Http404 += "<HR NOSHADE SIZE=1>\r\n";
 			Http404 += "Powered by <I><B><A HREF=\"http://mutella.sourceforge.net\">Mutella</A></B></I>.\r\n";
 			Http404 += "</BODY>\r\n";
@@ -384,8 +431,6 @@ void MGnuUpload::StatusUpdate(DWORD Status)
 
 	m_nStatus = Status;
 	m_nChangeTime = xtime();
-	
-	m_pDirector->TransferMessage(UPLOAD_UPDATE, (WPARAM) this);
 }
 
 void MGnuUpload::ForceDisconnect()
@@ -421,7 +466,7 @@ void MGnuUpload::PushFile()
 	StatusUpdate(TRANSFER_PUSH);
 	
 	// Set file name and size
-	if(!m_pShare->GetFile(m_nFileIndex, m_sFileName, m_sFilePath, false))
+	if(!m_pShare->GetFileByIndex(m_nFileIndex, m_sFileName, m_sFilePath, false))
 	{
 		m_nError = UPLERR_NOT_FOUND;
 		ForceDisconnect();
@@ -433,50 +478,112 @@ void MGnuUpload::PushFile()
 	m_pAFile->Open(m_sFilePath.c_str());
 }
 
-void MGnuUpload::UploadFile(CString lowHandshake, bool bLock)
+void MGnuUpload::UploadFile(const CString& sHandshake, bool bLock)
 {
 	//TRACE("MGnuUpload::UploadFile");
 	MLock lock(m_mutex, bLock);
 	ASSERT(m_mutex.locked());
 
-	lowHandshake.make_lower();
-	// Web browser compatibility
-	ReplaceSubStr(lowHandshake,"%20", " ");
-	
-	// Index
-	m_nFileIndex = -1;
-	CString FirstLine = lowHandshake.substr(0, lowHandshake.find("\r\n") );
-	
-	sscanf( FirstLine.c_str(), "get /get/%ld/", &m_nFileIndex);
-
-	// Filename
-	int Begin  = FirstLine.find("/", 9) + 1;
-	int	End    = FirstLine.find(" http/", Begin);
-	if (End < 0)
-		End = FirstLine.length();
-	m_sFileName = FirstLine.substr(Begin, End - Begin);
-	
+	// check for black-listed hosts
+	if (m_pPrefs->m_bBlackListActive && m_pPrefs->m_ipnetsetBlackList.IsIn(m_ipHost))
+	{
+		// we dont serve files to bustards
+		Send_HttpNotFound();
+		return;
+	}
 	// If there's an upload max
-	if(m_pPrefs->m_nMaxUploads >= 0 && m_pDirector->CountUploading() >= m_pPrefs->m_nMaxUploads)
+	if(m_pPrefs->m_nMaxUploads >= 0 && m_pDirector->CountUploads() >= m_pPrefs->m_nMaxUploads)
 	{
 		Send_HttpBusy();
 		return;
 	}
 
 	// If we are already uploading to this host, give a busy signal
-	if ( m_pPrefs->m_nMaxPerHostUploads >= 0 && m_pDirector->GetUploadsCount(m_ipHost, this) >= m_pPrefs->m_nMaxPerHostUploads )
+	if ( m_pPrefs->m_nMaxPerHostUploads > 0 && m_pDirector->CountUploads(m_ipHost, this) >= m_pPrefs->m_nMaxPerHostUploads )
 	{
 		Send_HttpBusy();
 		return;
 	}
+
+	CString sLowHandshake = sHandshake;
+	sLowHandshake.make_lower();
+
+	// or, process the request
+	if (0 == strncmp(sHandshake.c_str(),"GET /get/",9))
+	{
+		// we got traditional get request
+			
+		// Index
+		m_nFileIndex = -1;
+		CString FirstLine = sLowHandshake.substr(0, sLowHandshake.find("\r\n") );
 	
-	// Range
+		sscanf( FirstLine.c_str(), "get /get/%ld/", &m_nFileIndex);
+
+		// Filename
+		int Begin  = FirstLine.find("/", 9) + 1;
+		int	End    = FirstLine.find(" http/", Begin);
+		if (End < 0)
+			End = FirstLine.length();
+		m_sFileName = FirstLine.substr(Begin, End - Begin);
+		// Web browser compatibility
+		ReplaceSubStr(m_sFileName,"%20", " ");
+	}
+	else if (0 == strncmp(sHandshake.c_str(),"GET /uri-res/N2R?urn:sha1:",26))
+	{
+		// we got HUGE request
+		int nEnd = sHandshake.find(" HTTP/");
+		if (nEnd < 0)
+		{
+			Send_HttpNotFound();
+			return;
+		}
+		m_sSha1 = StripWhite(sHandshake.substr(26, nEnd-26));
+		if (m_sSha1.length() != 32)
+		{
+			Send_HttpNotFound();
+			return;
+		}
+		// resolve index from Sha1
+		m_nFileIndex = m_pShare->GetIndexBySha1(m_sSha1);
+		if (m_nFileIndex < 0)
+		{
+			Send_HttpNotFound();
+			return;
+		}
+
+		m_bUriRequest = true;
+	}
+	else
+	{
+		Send_HttpNotFound();
+		return;
+	}
+
+	// If we are already uploading the same file too many times
+	if ( m_pPrefs->m_nMaxPerFileUploads > 0 && m_pDirector->CountUploads(m_nFileIndex, this) >= m_pPrefs->m_nMaxPerFileUploads )
+	{
+		Send_HttpBusy();
+		return;
+	}
+
+    // User-Agent header
+	m_sUserAgent = find_header("User-Agent",sHandshake,sLowHandshake,"Unknown");
+	if (m_sUserAgent.length() > 25)
+	{
+		int nPos = m_sUserAgent.find(' ');
+		if (nPos>0)
+			m_sUserAgent.cut(nPos);
+	}
+
+	// Range header
 	m_nBytesCompleted = 0;
-	Begin = lowHandshake.find("\r\nrange: bytes=");
-	if(Begin > 0)
-	{	
-		CString sRange = lowHandshake.substr(Begin);
-		sscanf( sRange.c_str(), "\r\nrange: bytes=%ld-", &m_nBytesCompleted);
+	CString sRange = find_header("Range",sHandshake,sLowHandshake);
+	if(!sRange.empty())
+	{
+		sRange.make_lower();
+		sscanf( sRange.c_str(), "bytes=%ld-%ld", &m_nBytesCompleted, &m_nRequestedRange);
+		if (m_nRequestedRange>=0)
+			++m_nRequestedRange; // FIX: in HTTP/1.1 ranges are inclusive!
 	}
 	
 	// here compex part starts: if it is a push we have file already open, but it would make sence to verify if it is the right file.
@@ -484,7 +591,7 @@ void MGnuUpload::UploadFile(CString lowHandshake, bool bLock)
 	if (m_nStatus != TRANSFER_PUSH_CONNECTING )
 	{
 		// normal transfer
-		if(m_nFileIndex == -1 || !m_pShare->GetFile(m_nFileIndex, m_sFileName, m_sFilePath, true))
+		if(m_nFileIndex == -1 || !m_pShare->GetFileByIndex(m_nFileIndex, m_sFileName, m_sFilePath, m_sSha1.empty()))
 		{
 			m_nBytesCompleted = -1;
 			Send_HttpNotFound();
@@ -494,8 +601,11 @@ void MGnuUpload::UploadFile(CString lowHandshake, bool bLock)
 		m_pAFile->Open(m_sFilePath.c_str());
 		return;
 	}
+
+	// okm it's a push
 	// so the file is already open, lets check if it is the right one
-	if (m_nFileIndexPush != m_nFileIndex || !m_pAFile->IsOpen())
+	if (m_nFileIndexPush != m_nFileIndex ||
+		!m_pAFile->IsOpen()              )
 	{
 		TRACE("wrong push");
 		m_nBytesCompleted = -1;
@@ -506,8 +616,6 @@ void MGnuUpload::UploadFile(CString lowHandshake, bool bLock)
 	StatusUpdate(TRANSFER_CONNECTED);
 	// need to call OnFileOpen or do what is done there
 	OnFileOpen(false);
-	
-	m_pDirector->TransferMessage(UPLOAD_UPDATE, 0);
 }
 
 void MGnuUpload::OnFileOpen(bool bLock)
@@ -524,7 +632,7 @@ void MGnuUpload::OnFileOpen(bool bLock)
 		StatusUpdate(TRANSFER_PUSH_CONNECTING);
 		m_nFileIndexPush = m_nFileIndex;
 		m_nFileLength = m_pAFile->GetSize();
-		if(!Connect(IPtoStr(m_ipHost).c_str(), m_nPort))
+		if(!Connect(Ip2Str(m_ipHost).c_str(), m_nPort))
 			if(GetLastError() != EWOULDBLOCK && GetLastError() != EINPROGRESS)
 			{
 				m_nError = UPLERR_CONNECT_FAIL;
@@ -535,9 +643,15 @@ void MGnuUpload::OnFileOpen(bool bLock)
 	}
 
 	m_nFileLength = m_pAFile->GetSize();
+
+	if (m_nRequestedRange<=0)
+		m_nRequestedRange = m_nFileLength;
+	else if (m_nRequestedRange > m_nFileLength) // they made a mistake, we'll do what we can
+		m_nRequestedRange = m_nFileLength;
 	
-	if (m_nBytesCompleted < 0 || m_nBytesCompleted>=m_nFileLength)
+	if (m_nBytesCompleted < 0 || m_nBytesCompleted>=m_nRequestedRange)
 	{
+		// shall we send something to the server here, like bad request or something?
 		m_nError = UPLERR_BAD_REQUEST;
 		StatusUpdate(TRANSFER_CLOSED);
 		Close();
@@ -545,12 +659,21 @@ void MGnuUpload::OnFileOpen(bool bLock)
 	}
 	
 	Send_HttpOK();
+	m_bHttpOK = true; // we have accepted the transfer
 	
+	m_bBufferReady = false;
+	UpdateSelectFlags();
 	//TRACE("MGnuUpload::OnFileOpen -- calling PreFetch");
 	if(m_nBytesCompleted)
-		VERIFY(m_pAFile->PrefetchSeek(m_nBytesCompleted,SEEK_SET));
+	{
+		VERIFY(m_pAFile->ReadSeek(m_nBytesCompleted,SEEK_SET, m_pBuffer,
+				min(m_pBuffer->remains(),m_nRequestedRange - m_nBytesCompleted))>=0);
+	}
 	else
-		VERIFY(m_pAFile->Prefetch());
+	{
+		VERIFY(m_pAFile->Read(m_pBuffer, min(m_pBuffer->remains(),m_nRequestedRange - m_nBytesCompleted))>=0);
+	}
+	m_nStartOffset = m_nBytesCompleted;
 }
 
 void MGnuUpload::OnFileRead(int Requested, int nRead, int nError)
@@ -558,6 +681,10 @@ void MGnuUpload::OnFileRead(int Requested, int nRead, int nError)
 	//TRACE("MGnuUpload::OnFileRead");
 	
 	MLock lock(m_mutex);
+
+    ASSERT(m_pBuffer->isEmpty());
+    ASSERT(nRead > 0);
+	m_pBuffer->increment(nRead);
 	
 	m_bBufferReady = true;
 	UpdateSelectFlags();
@@ -566,10 +693,20 @@ void MGnuUpload::OnFileRead(int Requested, int nRead, int nError)
 void MGnuUpload::OnFileError(int nError)
 {
 	//TRACE("OnFileError");
-	MLock lock(m_mutex);
-	
+	MLock lock(m_mutex);	
 	if ( m_nStatus == TRANSFER_CONNECTED )
+	{
+		// we are just opening file and for what ever reason it did not work out...
 		Send_HttpNotFound();
+		// notify everybody who might be intrested
+		POST_EVENT( MIntEvent(
+			ET_ERROR,
+			ES_UNIMPORTANT,
+			m_nFileIndex,
+			MSG_UPLOAD_FILE_OPEN_FAILED,
+			MSGSRC_UPLOAD
+		));
+	}
 	// TODO: set m_nError for other cases correctly
 	ForceDisconnect();
 }
@@ -608,7 +745,7 @@ void MGnuUpload::BandwidthTimer()
 	{
 		m_nSecsDead++;
 
-		if(m_nSecsDead > 15)
+		if(m_nSecsDead > 60)
 		{
 			m_nError = UPLERR_NO_RESPONSE;
 			StatusUpdate(TRANSFER_CLOSED);
@@ -619,7 +756,7 @@ void MGnuUpload::BandwidthTimer()
 	{
 		m_nSecsDead++;
 
-		if(m_nSecsDead > 15)
+		if(m_nSecsDead > 60)
 		{
 			m_nError = UPLERR_CONNECT_FAIL;
 			StatusUpdate(TRANSFER_CLOSED);
@@ -669,6 +806,15 @@ void MGnuUpload::BandwidthTimer()
 			}
 		}
 	}
+	else if(TRANSFER_COMPLETED == m_nStatus)
+	{
+		m_nSecsDead++;
+		if(m_nSecsDead > 15)
+		{
+			ForceDisconnect();
+			m_nStatus = TRANSFER_CLOSED;
+		}
+	}
 	else if(TRANSFER_CLOSED == m_nStatus)
 	{
 	}
@@ -677,10 +823,14 @@ void MGnuUpload::BandwidthTimer()
 	m_nSecPos++;
 
 	// Check for completion
-	if(TRANSFER_SENDING == m_nStatus && ( m_nFileLength <= m_nBytesCompleted ))
+	//ASSERT(TRANSFER_SENDING != m_nStatus || m_nRequestedRange>0 );
+	if (!(TRANSFER_SENDING != m_nStatus || m_nRequestedRange>0 ))
+	{
+		cout << m_sHandshake << endl;
+		ASSERT(0);
+	}
+	if(TRANSFER_SENDING == m_nStatus && ( m_nRequestedRange <= m_nBytesCompleted ))
 	{
-		ASSERT(m_nFileLength);
-		//m_pDirector->m_pController->SetHaveUploaded(true);
 		Close();
 		StatusUpdate(TRANSFER_COMPLETED);
 	}
@@ -692,32 +842,58 @@ void MGnuUpload::OnSend(int nErrorCode)
 	MLock lock(m_mutex);
 	
 	ASSERT(m_pAFile);
-	if (!m_pAFile->BytesAvail())
+	ASSERT(m_pBuffer != NULL);
+	if (!m_pBuffer->contains())
 	{
-		TRACE("WARNING: MGnuUpload::OnSend occured when no data is available");
+		TRACE( "WARNING: MGnuUpload::OnSend occured when no data is available");
+		TRACE2("         m_nBytesCompleted                      = ", m_nBytesCompleted);
+		TRACE2("         m_nFileLength                          = ", m_nFileLength);
+		TRACE2("         m_bBufferReady                         = ", m_bBufferReady);
+		TRACE2("         m_pAFile->GetPos()                     = ", m_pAFile->GetPos());
+		TRACE2("         m_pAFile->GetAsyncPos()                = ", m_pAFile->GetPos());
+		TRACE2("         m_pAFile->GetCurrRequest()             = ", m_pAFile->GetCurrRequest());
+		TRACE2("         m_pAFile->GetCurrComplRequest()        = ", m_pAFile->GetCurrComplRequest());
+		TRACE2("         m_pAFile->GetCurrDelayedComplRequest() = ", m_pAFile->GetCurrDelayedComplRequest());
+
 		return ; // quick and dirty
 	}
 	if (m_nStatus != TRANSFER_SENDING)
+	{
+		TRACE("WARNING: MGnuUpload::OnSend occured when m_nStatus != TRANSFER_SENDING");
 		return ; // quick and dirty
-	if(m_nBytesCompleted >= m_nFileLength)
+	}
+	ASSERT(m_nRequestedRange>0);
+	if(m_nBytesCompleted >= m_nRequestedRange)
+	{
+		TRACE("WARNING: MGnuUpload::OnSend encounetered m_nBytesCompleted >= m_nRequestedRange situation");
 		return ; // quick and dirty
+	}
+	if (!m_bBufferReady)
+	{
+		TRACE ("WARNING: MGnuUpload::OnSend happen when m_bBufferReady = false");
+		return ; // quick and dirty
+	}
 		
 	// we have what to send -- do it
-	int nBytesAvail = m_pAFile->BytesAvail();
-	int nBytesSentNow = Send(m_pAFile->GetBuffer(), nBytesAvail);
+	int nBytesAvail = m_pBuffer->contains();
+	int nBytesSentNow = Send(m_pBuffer->data(), nBytesAvail);
 	if (nBytesSentNow <= 0)
 	{
-		m_pAFile->ReleaseBuffer(0);
+		m_pBuffer->clear();
 		ForceDisconnect();
 		m_nStatus = TRANSFER_CLOSED;
 		m_nError = UPLERR_SOCKET_EROR;
 		return;
 	}
-	VERIFY(m_pAFile->ReleaseBuffer(nBytesSentNow));
+	if (nBytesSentNow == nBytesAvail)
+		m_pBuffer->clear();
+	else
+	{
+		VERIFY(m_pBuffer->offset(nBytesSentNow));
+	}
 	// total count, statistics, etc
 	m_dwSecBytes += nBytesSentNow;
 	m_nBytesCompleted += nBytesSentNow;
-	ASSERT(m_pAFile->GetPos()==m_nBytesCompleted);
 	// limit bandwidth
 	if (m_dMaxRate>0 && m_dwSecBytes>m_dMaxRate && m_bBelowRateLimit)
 	{
@@ -725,17 +901,21 @@ void MGnuUpload::OnSend(int nErrorCode)
 		UpdateSelectFlags();
 	}
 	// check for completion
-	if (m_nBytesCompleted == m_nFileLength)
+	ASSERT(m_nRequestedRange>0);
+	if (m_nBytesCompleted >= m_nRequestedRange)
 	{
-		 ForceDisconnect();
+		 //ForceDisconnect();
+		 ModifySelectFlags(0,FD_WRITE);
 		 m_nStatus = TRANSFER_COMPLETED;
 	}
-	//
-	if (!m_pAFile->BytesAvail())
+	else
+	if (!m_pBuffer->contains())
 	{
+		ASSERT(m_pAFile->GetPos()==m_nBytesCompleted);
 		m_bBufferReady = false;
 		UpdateSelectFlags();
-		VERIFY(m_pAFile->Prefetch());
+		ASSERT(m_pBuffer->remains()==4096);
+		m_pAFile->Read(m_pBuffer, min(m_pBuffer->remains(),m_nRequestedRange - m_nBytesCompleted));
 	}
 	MAsyncSocket::OnSend(nErrorCode);
 }
