@@ -1,3 +1,25 @@
+/***************************************************************************
+ Mutella - A commandline/HTTP client for the Gnutella filesharing network.
+
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License
+ as published by the Free Software Foundation; either version 2
+ of the License, or (at your option) any later version.
+
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+
+ inifile.cpp  -  INI file read/write capabilities - preferences storage format
+
+ original version of this file was taken from LICQ program (http://www.licq.org)
+
+    begin                : Wed May 30 2001
+    copyright            : (C) 2001 by
+    email                : maksik@gmx.co.uk
+ ***************************************************************************/
+ 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -10,11 +32,13 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <ctype.h>
+#include <assert.h>
 
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
 
+#include "defines.h"
 #include "inifile.h"
 
 //=====Pre class helper functions==============================================
@@ -162,6 +186,7 @@ bool MIniFile::LoadFile(const char *_szFilename)
 {
   CloseFile();
   m_szFilename = strdup(_szFilename);
+  ASSERT(m_szFilename);
 
   int nFD;
   if (GetFlag(INI_FxALLOWxCREATE))
@@ -182,6 +207,7 @@ bool MIniFile::LoadFile(const char *_szFilename)
   struct stat sFileStats;
   fstat(nFD, &sFileStats);
   m_szBuffer = (char *)malloc((unsigned int)(sFileStats.st_size + 2));
+  ASSERT(m_szBuffer);
   m_nBufSize = read(nFD, m_szBuffer, (unsigned int)sFileStats.st_size);
   close (nFD);
 
@@ -274,6 +300,7 @@ void MIniFile::SetFileName(const char *_szFilename)
 {
   if (m_szFilename != NULL) free(m_szFilename);
   m_szFilename = strdup(_szFilename);
+  ASSERT(m_szFilename);
 }
 
 
@@ -536,6 +563,7 @@ bool MIniFile::SetSection(const char *_szSection)
 
   m_nSectionEnd = nTempPos + 1;
   m_szSectionName = strdup(_szSection);
+  ASSERT(m_szSectionName);
   ResetSection();
   return (true);
 }
@@ -690,6 +718,7 @@ void MIniFile::InsertStr(const char *_szNewStr, int _nCutStart, int _nCutEnd)
   int nNewBufSize = _nCutStart + nNewStrLen + m_nBufSize - _nCutEnd;
 
   char *szNewBuffer = (char *)malloc(nNewBufSize + 1);
+  ASSERT(szNewBuffer);
   memcpy(szNewBuffer, m_szBuffer, _nCutStart);
   memcpy(szNewBuffer + _nCutStart, _szNewStr, nNewStrLen);
   memcpy(szNewBuffer + _nCutStart + nNewStrLen, m_szBuffer + _nCutEnd, m_nBufSize - _nCutEnd + 1);
@@ -726,6 +755,7 @@ bool MIniFile::CreateSection(const char *_szSectionName)
   InsertStr(szNewSect, m_nBufPos, m_nBufPos);
   m_nSectionEnd = m_nBufSize;
   m_szSectionName = strdup(_szSectionName);
+  ASSERT(m_szSectionName);
 
   return(true);
 }
