@@ -1,6 +1,18 @@
 /***************************************************************************
-                          packet.h  -  description
-                             -------------------
+ Mutella - A commandline/HTTP client for the Gnutella filesharing network.
+
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License
+ as published by the Free Software Foundation; either version 2
+ of the License, or (at your option) any later version.
+
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+
+ tstring.h  -  STL-like string class, for portability.
+ 
     begin                : Thu May 31 2001
     copyright            : (C) 2001 by
     email                : maksik@gmx.co.uk
@@ -10,6 +22,7 @@
 #define __TSTRING_H__INCLUDED__
 
 #include <stdarg.h>
+#include <iostream>
 
 #ifdef TSTRING_DEBUG
 
@@ -75,12 +88,43 @@ public:
 	}
 	const TString<T>& operator=(T c) {
 		CHECK_CONSIST
-		if (cRhs == _null) return *this;
+		if (c == _null){
+			clear();
+			return *this;
+		}
 		T str[2] = { c, _null };
 		_reallocNreplace(1, str);
 		CHECK_CONSIST
 		return *this;
 	}
+	// assignments of partial strings
+	void assign(const TString<T>& orig, int nSize){
+		CHECK_CONSIST
+		_reallocNreplace(_min(orig.m_nLength, nSize), orig.m_pBuffer);
+		CHECK_CONSIST
+	}
+	void assign(LPCT psz,int nSize){
+		CHECK_CONSIST
+		_reallocNreplace(_strnlen(psz,nSize), psz);
+		CHECK_CONSIST
+	}
+	void assign(const TString<T>& orig, int nStart, int nLen) {
+		if (nStart >= orig.m_nLength) return;
+		if (nLen > orig.m_nLength-nStart) nLen = orig.m_nLength-nStart;
+		if (nLen <= 0) return;
+		CHECK_CONSIST
+		_reallocNreplace( nLen, orig.m_pBuffer+nStart);
+		CHECK_CONSIST
+	}
+	void assign(LPCT psz, int nStart, int nLen){
+		int nLenOrig = _strnlen(psz,nStart+nLen);
+		if (nStart >= nLenOrig) return;
+		if (nLen > nLenOrig-nStart) nLen = nLenOrig-nStart;
+		if (nLen <= 0) return;
+		CHECK_CONSIST
+		_reallocNreplace( nLen, psz+nStart);
+		CHECK_CONSIST
+	}
 	// concatetation
 	const TString<T>& operator+=(const TString<T>& rhs) {
 		CHECK_CONSIST
@@ -115,13 +159,23 @@ public:
 		CHECK_CONSIST
 		return m_nBufferSize;
 	}
+	int capacity() const {
+		CHECK_CONSIST
+		return m_nBufferSize-1;
+	}
+	void reserve(int nMaxLength){
+		CHECK_CONSIST
+		if (nMaxLength+1>m_nBufferSize)
+			_realloc(nMaxLength);
+	}
 	bool empty() const {
 		CHECK_CONSIST
 		return 0==m_nLength;
 	}
 	void clear(){
 		CHECK_CONSIST
-		delete [] m_pBuffer;
+		//delete [] m_pBuffer;
+		free(m_pBuffer);
 		m_pBuffer = NULL;
 		m_nBufferSize = 0;
 		m_nLength = 0;
@@ -139,6 +193,51 @@ public:
 				CHECK_CONSIST
 			}
 	}
+	TString<T>& insert(int nPos, const TString<T>& s) {
+		CHECK_CONSIST
+		if (nPos > m_nLength)
+			nPos = m_nLength;
+		LPT pOldBuf = m_pBuffer;
+		int nOldLen = m_nLength;
+		// fake clear()
+		m_nBufferSize = 0;
+		m_nLength = 0;
+		m_pBuffer = NULL;
+		//
+		_reallocNreplace(nOldLen+s.m_nLength, NULL);
+		memcpy(m_pBuffer, pOldBuf, sizeof(T)*nPos);
+		memcpy(m_pBuffer+nPos, s.m_pBuffer, sizeof(T)*s.m_nLength);
+		memcpy(m_pBuffer+(nPos+s.m_nLength), pOldBuf+nPos, sizeof(T)*(nOldLen-nPos));
+		m_nLength = nOldLen+s.m_nLength;
+		m_pBuffer[m_nLength] = _null;
+		CHECK_CONSIST
+		//
+		free(pOldBuf);
+		return *this;
+	}
+	TString<T>& insert(int nPos, LPCT psz) {
+		CHECK_CONSIST
+		if (nPos > m_nLength)
+			nPos = m_nLength;
+		int nAddLen = _strlen(psz);
+		LPT pOldBuf = m_pBuffer;
+		int nOldLen = m_nLength;
+		// fake clear()
+		m_nBufferSize = 0;
+		m_nLength = 0;
+		m_pBuffer = NULL;
+		//
+		_reallocNreplace(nOldLen+nAddLen, NULL);
+		memcpy(m_pBuffer, pOldBuf, sizeof(T)*nPos);
+		memcpy(m_pBuffer+nPos, psz, sizeof(T)*nAddLen);
+		memcpy(m_pBuffer+(nPos+nAddLen), pOldBuf+nPos, sizeof(T)*(nOldLen-nPos));
+		m_nLength = nOldLen+nAddLen;
+		m_pBuffer[m_nLength] = _null;
+		CHECK_CONSIST
+		//
+		free(pOldBuf);
+		return *this;
+	}
 	// search
 	int find(const TString<T>& needle, int nStart=0) const {
 		CHECK_CONSIST
@@ -232,7 +331,7 @@ public:
 		return m_pBuffer[i];
 	}
 	// sprintf replacement
-	inline const TString<T>& format(const char *fmt, ...){
+	TString<T>& format(const char *fmt, ...){
 		va_list ap;
 		va_start(ap, fmt);
 		format(fmt, ap);
@@ -247,7 +346,14 @@ public:
 			// Ensure we hawe that much space in the buffer
 			_reallocNreplace(size, NULL);
 			// Try to print in the allocated space
+#ifdef VA_COPY
+			va_list tmp;      // required for PPC
+			VA_COPY(tmp, ap); // required for PPC
+			n = _vsnprintf(m_pBuffer, m_nBufferSize, fmt, tmp);
+			va_end(ap);       // required for PPC
+#else //VA_COPY
 			n = _vsnprintf(m_pBuffer, m_nBufferSize, fmt, ap);
+#endif //VA_COPY
 			// If that worked, return the string
 			if (n > -1 && n < m_nBufferSize)
 			{
@@ -264,7 +370,58 @@ public:
 	};
 protected:
 	// methods
+	void _realloc(int nLength) { // allocate buffer to hold string of length nLength (no optimisation)
+#ifdef CHECK_RANGE
+		if (nLength < 0)
+			fprintf(stderr, "TString::_reallocNreplace nLength < 0\n");
+#endif
+		// TODO: shrink buffer if the string is tooo short (check usage of this function before)
+		if (nLength+1>m_nBufferSize) {
+			m_nBufferSize = nLength+1; // realloc is exact function
+			m_pBuffer = (T*) realloc(m_pBuffer, sizeof(T)*m_nBufferSize);
+			ASSERT(m_pBuffer);
+		}
+	}
 	void _reallocNtail(int nAdd, LPCT pTail = NULL){
+#ifdef CHECK_RANGE
+		if ( nAdd < 0 )
+			fprintf(stderr, "TString::_reallocNtail nAdd < 0\n");
+#endif
+		if (m_nLength+1+nAdd>m_nBufferSize){
+			_realloc(m_nLength+nAdd+16); // TODO: optimize reallocations
+		}
+		if (pTail) {
+#ifdef CHECK_RANGE
+				if (nAdd > _strnlen(pTail, nAdd))
+					fprintf(stderr, "TString::_reallocNtail nAdd > _strlen(pTail)\n");
+#endif
+				memcpy(m_pBuffer+m_nLength, pTail, sizeof(T)*nAdd); // here we trust that nAdd <= strlen(pTail)
+				m_nLength += nAdd;
+				m_pBuffer[m_nLength] = _null;
+		}
+	}
+	void _reallocNreplace(int nLength, LPCT pContent = NULL ) { // allocate buffer to hold string of length nLength
+#ifdef CHECK_RANGE
+		if (nLength < 0)
+			fprintf(stderr, "TString::_reallocNreplace nLength < 0\n");
+#endif
+		// TODO: shrink buffer if the string is tooo short (check usage of this function before)
+		if (nLength+1>m_nBufferSize) {
+			_realloc(nLength+16); // TODO: optimize this +16
+		}
+		if (pContent) {
+#ifdef CHECK_RANGE
+			if (nLength > _strnlen(pContent, nLength))
+				fprintf(stderr, "TString::_reallocNreplace nLength > _strlen(pContent)\n");
+#endif
+			memcpy(m_pBuffer, pContent, sizeof(T)*nLength); // here we trust that nLength <= strlen(pContent)
+			m_nLength = nLength;
+		} else {
+			m_nLength = 0;
+		}
+		m_pBuffer[m_nLength] = _null;
+	}
+/*	void _reallocNtail(int nAdd, LPCT pTail = NULL){
 #ifdef CHECK_RANGE
 		if ( nAdd < 0 )
 			fprintf(stderr, "TString::_reallocNtail nAdd < 0\n");
@@ -333,7 +490,7 @@ protected:
 			m_nLength = 0;
 		}
 		m_pBuffer[m_nLength] = _null;
-	}
+	}*/
 	// helpers
 	int _min(register int a, register int b) { if ( a<b ) return a; return b; }
 	int _max(register int a, register int b) { if ( a>b ) return a; return b; }
@@ -350,6 +507,7 @@ protected:
 #endif
 	static int _strnlen(LPCT psz, int nMax){
 		if (!psz) return 0;
+		if (nMax<0 || nMax==INT_MAX ) return _strlen(psz);
 		for (int i = 0; i<nMax; i++)
 			if (psz[i] == _null)
 				return i;
@@ -396,8 +554,8 @@ protected:
 	friend TString<T> operator+ <>(typename TString<T>::LPCT pszLhs, const TString<T>& rhs);
 	friend TString<T> operator+ <>(const TString<T>& lhs, T cRhs);
 	friend TString<T> operator+ <>(T cLhs, const TString<T>& rhs);
-	friend ostream& operator<< <>(ostream& os, const TString<T>& st);
-	friend istream& operator>> <>(istream& is, const TString<T>& st);
+	friend std::ostream& operator<< <>(std::ostream& os, const TString<T>& st);
+	friend std::istream& operator>> <>(std::istream& is, const TString<T>& st);
 };
 
 template<class T>
@@ -515,8 +673,8 @@ TString<T> operator+(const TString<T>& lhs, T cRhs) {
 	TString<T> res;
 	res._reallocNreplace(lhs.m_nLength + 1);
 	res._reallocNreplace(lhs.m_nLength, lhs.m_pBuffer);
-	if (cRhs != _null){
-		T str[2] = { cRhs, _null };
+	if (cRhs != TString<T>::_null){
+		T str[2] = { cRhs, TString<T>::_null };
 		res._reallocNtail(1,str);
 	}
 	return res;
@@ -525,8 +683,8 @@ template<class T>
 TString<T> operator+(T cLhs, const TString<T>& rhs) {
 	TString<T> res;
 	res._reallocNreplace(1 + rhs.m_nLength);
-	if (cLhs != _null){
-		T str[2] = { cLhs, _null };
+	if (cLhs != TString<T>::_null){
+		T str[2] = { cLhs, TString<T>::_null };
 		res._reallocNreplace(1,str);
 	}
 	res._reallocNtail(rhs.m_nLength, rhs.m_pBuffer);
@@ -534,12 +692,12 @@ TString<T> operator+(T cLhs, const TString<T>& rhs) {
 }
 
 template<class T>
-ostream& operator<<(ostream& os, const TString<T>& st) {
+std::ostream& operator<<(std::ostream& os, const TString<T>& st) {
 	return os << st.m_pBuffer;
 }
 
 template<class T>
-istream& operator>>(istream& is, const TString<T>& st){
+std::istream& operator>>(std::istream& is, const TString<T>& st){
 	ASSERT(0);// not implemented properly
 	return is >> st.m_pBuffer;
 }
